#!/usr/bin/python
# coding=UTF-8
from __future__ import (unicode_literals, division, absolute_import,
                        print_function)
import os
import sys

class EncodingFixer(object):
    def __init__(self, table):
        self.b = bytearray(1)
        if sys.version_info[0] == 3:
            self.char0 = self.char0_v3
        else:
            self.char0 = self.char0_v2
        self.createlookupTable(table)
        self.changed = False


    def createlookupTable(self, table):
        self.lookupTable = {}
        for i in table:
            k1 = i >> 8
            k2 = i & 0xFF
            if k1 not in self.lookupTable:
                self.lookupTable[k1] = {}
            self.lookupTable[k1][k2] = table[i].encode("UTF-8")

    def char0_v2(self, someString):  # for Python 2.7
        return ord(someString[0])

    def char0_v3(self, someByteArray):  # for Pyton 3.*
        return someByteArray[0]


    def printVal(self, value):
        self.b[0] = value
        self.fo.write(self.b)

    def printBytes(self, array):
        self.changed = True
        for val in array:
            self.b[0] = val
            self.fo.write(self.b)

    def run(self, fh, fo):
        self.fh = fh
        self.fo = fo

        last = 0
        while 1:
            # read character by character
            character = self.fh.read(1)   # in Python 2.* it returns string, in 3.* bytearray
            if not character:
                break
            o1 = self.char0(character)

            if o1 in self.lookupTable:
                character = self.fh.read(1)
                if not character:
                    break
                o2 = self.char0(character)
                if o2 in self.lookupTable[o1]:
                    # 'last' was the first byte of multi byte UTF sequence,
                    # so it is a bad idea to replace the following bytes
                    #  see the table on http://en.wikipedia.org/wiki/UTF-8#Design
                    #~ if (last & 0xE0) > 0xC0:
                        #~ print("Something bad happened")
                    replace = self.lookupTable[o1][o2]
                    self.printBytes(replace)
                else:
                    self.printVal(o1)
                    self.printVal(o2)
                last = o2
            else:
                self.printVal(o1)
                last = o1
        return self.changed


if __name__ == "__main__":
# You can use this file as a standalone python script
# Usage: ./main.py infile outfile
# the table is based on a PDF file which I tried to correct, just for demonstartion purpose
# you can replace it with your own
    table = {
        0xc3a8: "č",
        0xc388: "Č",
        0xc3AF: "ď",
        0xc38c: "Ě",
        0xc3ac: "ě",
        0xc392: "Ň",
        0xc3b2: "ň",
        0xc398: "Ř",
        0xc3b8: "ř",
        0xc28a: "Š",
        0xc29a: "š",
        0xc29d: "ť",
        0xc399: "Ů",
        0xc3b9: "ů",
        0xc29e: "ž",
        0xc28e: "Ž",
        0xc284: "„",
        0xc291: "‘",
        0xc292: "’",
        0xc293: "”",
        0xc294: "”",
        0xc296: "-",
    }
# not covered:
# á, Á - seems to be encoded well
# Ď - not found
# é, É  - seems to be encoded well
# Í, í - seems to be encoded well
# Ľ, ľ - not found  (Slovak letter)
# ó - seems to be encoded well
# Ó - not found
# Ť  - not found
# ú, Ú - seems to be encoded well
# ý, Ý - seems to be encoded well


    fh = open(sys.argv[1], "rb")
    fo = open(sys.argv[2], "wb")
    fixer = EncodingFixer(table)
    fixer.run(fh, fo)
    fo.close()
